#include <stdexcept>
#include <msp/strings/format.h>
#include "glcontext.h"
#include "window.h"

using namespace std;

namespace Msp {
namespace Graphics {

GLOptions::GLOptions():
	alpha(false),
	stencil(false),
	doublebuffer(true),
	multisample(0),
	forward_compatible(false),
	core_profile(false),
	gl_version_major(DEFAULT_VERSION),
	gl_version_minor(DEFAULT_VERSION)
{ }


unsupported_gl_mode::unsupported_gl_mode(const GLOptions &opts):
	runtime_error(format("{ .alpha=%s, .stencil=%s, .doublebuffer=%s, .multisample=%d, .forward_compatible=%s, .core_profile=%s, .gl_version=%s }",
		opts.alpha, opts.stencil, opts.doublebuffer, opts.multisample, opts.forward_compatible, opts.core_profile, format_version(opts)))
{ }

string unsupported_gl_mode::format_version(const GLOptions &opts)
{
	if(opts.gl_version_major==GLOptions::LATEST_VERSION)
		return "LATEST";
	else if(opts.gl_version_major==GLOptions::DEFAULT_VERSION)
		return "DEFAULT";
	else
		return format("%d.%d", opts.gl_version_major, opts.gl_version_minor);
}



GLContext::GLContext(Window &wnd, const GLOptions &opts):
	display(wnd.get_display()),
	window(wnd)
{
	platform_init(opts);

	window.signal_resize.connect(sigc::mem_fun(this, &GLContext::window_resized));
}

#ifndef WITH_OPENGL
void GLContext::platform_init(const GLOptions &)
{
	throw runtime_error("no OpenGL support");
}

GLContext::~GLContext()
{ }

void GLContext::set_swap_interval(unsigned)
{ }

void GLContext::swap_buffers()
{ }

void GLContext::window_resized(unsigned, unsigned)
{ }
#endif

} // namespace Graphics
} // namespace Msp
