#ifndef MSP_GRAPHICS_IMAGE_H_
#define MSP_GRAPHICS_IMAGE_H_

#include <stdexcept>
#include <string>
#include <msp/io/seekable.h>
#include "pixelformat.h"

namespace Msp {
namespace Graphics {

class ImageLoader;

class Image
{
public:
	struct Data
	{
		PixelFormat fmt;
		unsigned width;
		unsigned height;
		unsigned stride;
		char *owned_pixels;
		char *pixels;

		Data();
		Data(const Data &);
		Data &operator=(const Data &);
		~Data();
	};

private:
	Data data;

public:
	void load_file(const std::string &);
	void load_io(IO::Seekable &);
	void load(ImageLoader &);
	void load_into(ImageLoader &, void *);
	void load_headers(ImageLoader &);

	PixelFormat get_format() const { return data.fmt; }
	unsigned get_width() const { return data.width; }
	unsigned get_height() const { return data.height; }
	unsigned get_stride() const { return data.stride; }
	DEPRECATED const void *get_data() const { return data.pixels; }
	const void *get_pixels() const { return data.pixels; }
};

} // namespace Graphics
} // namespace Msp

#endif
