#include <algorithm>
#include <ApplicationServices/ApplicationServices.h>
// Avoid messing up sigc++ headers
#undef nil
#include "quartzloader.h"

using namespace std;

namespace {

size_t get_bytes(void *info, void *buffer, size_t count)
{
	Msp::IO::Seekable *io = reinterpret_cast<Msp::IO::Seekable *>(info);
	char *cbuf = reinterpret_cast<char *>(buffer);
	return io->read(cbuf, count);
}

off_t skip_forward(void *info, off_t count)
{
	Msp::IO::Seekable *io = reinterpret_cast<Msp::IO::Seekable *>(info);

	off_t i = 0;
	char buf[1024];
	while(i<count)
		i += io->read(buf, min<unsigned>(sizeof(buf), count-i));

	return i;
}

void rewind(void *info)
{
	Msp::IO::Seekable *io = reinterpret_cast<Msp::IO::Seekable *>(info);
	io->seek(0, Msp::IO::S_BEG);
}

CGDataProviderSequentialCallbacks callbacks =
{
	0,
	&get_bytes,
	&skip_forward,
	&rewind,
	0
};

} // namespace


namespace Msp {
namespace Graphics {

struct QuartzLoader::Private
{
	CGDataProviderRef provider;
	CGImageSourceRef source;
};


QuartzLoader::QuartzLoader(IO::Seekable &io):
	priv(new Private)
{
	priv->provider = CGDataProviderCreateSequential(&io, &callbacks);
	priv->source = CGImageSourceCreateWithDataProvider(priv->provider, 0);
}

QuartzLoader::~QuartzLoader()
{
	CFRelease(priv->source);
	CFRelease(priv->provider);
	delete priv;
}

bool QuartzLoader::detect(const string &sig)
{
	CGImageSourceRef source = CGImageSourceCreateIncremental(0);
	CFDataRef data = CFDataCreate(0, reinterpret_cast<const UInt8 *>(sig.data()), sig.size());
	CGImageSourceUpdateData(source, data, false);
	CGImageSourceStatus status = CGImageSourceGetStatus(source);
	CFRelease(data);
	CFRelease(source);

	return status==kCGImageStatusIncomplete || status==kCGImageStatusReadingHeader;
}

void QuartzLoader::load_(Image::Data &data)
{
	CGImageRef image = CGImageSourceCreateImageAtIndex(priv->source, 0, 0);
	if(!image)
		throw bad_image_data("null image");

	try
	{
		data.width = CGImageGetWidth(image);
		data.height = CGImageGetHeight(image);
		data.stride = CGImageGetBytesPerRow(image);

		CGColorSpaceRef color = CGImageGetColorSpace(image);
		CGColorSpaceModel model = CGColorSpaceGetModel(color);
		CGImageAlphaInfo alpha = CGImageGetAlphaInfo(image);
		if(model==kCGColorSpaceModelRGB)
		{
			if(alpha==kCGImageAlphaLast)
				data.fmt = RGBA;
			else if(alpha==kCGImageAlphaNoneSkipLast || alpha==kCGImageAlphaNoneSkipFirst)
				data.fmt = RGBX;
			else if(alpha==kCGImageAlphaNone)
				data.fmt = RGB;
			else
				throw unsupported_image_format("unknown alpha mode");
		}
		else
			throw unsupported_image_format("unknown colorspace");
	}
	catch(...)
	{
		CFRelease(image);
		throw;
	}
}

void QuartzLoader::load_(Image::Data &data)
{
	CGDataProviderRef dp = CGImageGetDataProvider(image);
	CFDataRef image_data = CGDataProviderCopyData(dp);
	unsigned offset = (alpha==kCGImageAlphaNoneSkipFirst);
	CFRange range = CFRangeMake(offset, CFDataGetLength(image_data)-offset);
	CFDataGetBytes(image_data, range, reinterpret_cast<UInt8 *>(data.pixels));
	CFRelease(image_data);

	CFRelease(image);
}

} // namespace Graphics
} // namespace Msp
